@@ -10,12 +10,13 @@
 from __future__ import print_function
 
 import io
+import os
 import sys
+import tempfile
 import unittest
 
-from pygments import highlight
+from pygments import highlight, cmdline
 from pygments.util import StringIO, BytesIO
-from pygments.cmdline import main as cmdline_main
 
 import support
 
@@ -34,7 +35,7 @@ def run_cmdline(*args):
         stdout_buffer = new_stdout = sys.stdout = StringIO()
         stderr_buffer = new_stderr = sys.stderr = StringIO()
     try:
-        ret = cmdline_main(["pygmentize"] + list(args))
+        ret = cmdline.main(["pygmentize"] + list(args))
     finally:
         sys.stdout = saved_stdout
         sys.stderr = saved_stderr
@@ -111,3 +112,54 @@ class CmdLineTest(unittest.TestCase):
         self.assertEqual(o, output)
         self.assertEqual(e, "")
         self.assertEqual(c, 0)
+
+    def test_outfile(self):
+        # test that output file works with and without encoding
+        fd, name = tempfile.mkstemp()
+        os.close(fd)
+        for opts in [['-fhtml', '-o', name, TESTFILE],
+                     ['-flatex', '-o', name, TESTFILE],
+                     ['-fhtml', '-o', name, '-O', 'encoding=utf-8', TESTFILE]]:
+            try:
+                self.assertEqual(run_cmdline(*opts)[0], 0)
+            finally:
+                os.unlink(name)
+
+    def check_failure(self, *cmdline):
+        c, o, e = run_cmdline(*cmdline)
+        self.assertEqual(c, 1)
+        self.assertEqual(o, '')
+        return e
+
+    def test_errors(self):
+        # input file not found
+        e = self.check_failure('-lpython', 'nonexistent.py')
+        self.assertTrue('Error: cannot read infile' in e)
+        self.assertTrue('nonexistent.py' in e)
+
+        # lexer not found
+        e = self.check_failure('-lfooo', TESTFILE)
+        self.assertTrue('Error: no lexer for alias' in e)
+
+        # formatter not found
+        e = self.check_failure('-lpython', '-ffoo', TESTFILE)
+        self.assertTrue('Error: no formatter found for name' in e)
+
+        # output file not writable
+        e = self.check_failure('-o', os.path.join('nonexistent', 'dir', 'out.html'),
+                               '-lpython', TESTFILE)
+        self.assertTrue('Error: cannot open outfile' in e)
+        self.assertTrue('out.html' in e)
+
+        # unknown filter
+        e = self.check_failure('-F', 'foo', TESTFILE)
+        self.assertTrue('Error: filter \'foo\' not found' in e)
+
+    def test_exception(self):
+        # unexpected exception while highlighting
+        cmdline.highlight = None  # override callable
+        try:
+            e = self.check_failure('-lpython', TESTFILE)
+        finally:
+            cmdline.highlight = highlight
+        self.assertTrue('*** Error while highlighting:' in e)
